'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _pauseable = require('pauseable');

var _pauseable2 = _interopRequireDefault(_pauseable);

var _yamljs = require('yamljs');

var _yamljs2 = _interopRequireDefault(_yamljs);

var _http = require('http');

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _bluebird = require('bluebird');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _md = require('md5');

var _md2 = _interopRequireDefault(_md);

var _chokidar = require('chokidar');

var _chokidar2 = _interopRequireDefault(_chokidar);

var _dispatcher = require('../dispatcher');

var _dispatcher2 = _interopRequireDefault(_dispatcher);

var _server = require('./server');

var _server2 = _interopRequireDefault(_server);

var _datadir = require('../validators/datadir');

var _datadir2 = _interopRequireDefault(_datadir);

var _log = require('../log');

var _log2 = _interopRequireDefault(_log);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PORT = 35590;

var fs = _bluebird.Promise.promisifyAll(require('fs'));

var OtaServer = function (_Server) {
  (0, _inherits3.default)(OtaServer, _Server);

  function OtaServer(options) {
    var _this2 = this;

    (0, _classCallCheck3.default)(this, OtaServer);

    var _this = (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(OtaServer).call(this, 'OTA'));

    _this.dataDir = options.dataDir;
    _this.manifestPath = _path2.default.join(_this.dataDir, '/ota/manifest.yml');

    _this.app = (0, _express2.default)();
    _this.server = (0, _http.createServer)(_this.app);
    _this.app.use('/ota', _this._httpHandler);

    var watcher = _chokidar2.default.watch(_this.manifestPath);
    _pauseable2.default.pause(watcher); // Buffer events until dispatcher is ready
    _dispatcher2.default.on('ready', function () {
      _pauseable2.default.resume(watcher);

      _this._loadManifest().then(function (manifest) {
        _this.manifest = manifest;
        if (_this.manifest) {
          _this._warnDevices();
        }
      });
    });

    watcher.on('change', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _log2.default.info('OTA manifest updated');

              _this._loadManifest().then(function (manifest) {
                _this.manifest = manifest;
                if (_this.manifest) {
                  _this._warnDevices();
                }
              });

            case 2:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this2);
    })));
    return _this;
  }

  (0, _createClass3.default)(OtaServer, [{
    key: 'start',
    value: function start() {
      var _this3 = this;

      return new _bluebird.Promise(function (resolve, reject) {
        _this3.server.listen(PORT, function () {
          var host = _this3.server.address().address;
          var port = _this3.server.address().port;
          resolve({ host: host, port: port });
        }).on('error', function (err) {
          reject(err);
        });
      });
    }
  }, {
    key: '_warnDevices',
    value: function _warnDevices() {
      this.manifest.firmwares.forEach(function (firmware) {
        firmware.devices.forEach(function (device) {
          _dispatcher2.default.publishMqttMessage({
            cmd: 'publish',
            topic: 'devices/' + device + '/$ota',
            payload: firmware.version.toString(),
            qos: 2,
            retain: true
          });
        });
      });
    }
  }, {
    key: '_loadManifest',
    value: function () {
      var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
        var rawManifest, manifest, valid;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.prev = 0;
                _context2.next = 3;
                return fs.readFileAsync(this.manifestPath);

              case 3:
                rawManifest = _context2.sent;
                manifest = _yamljs2.default.parse(rawManifest.toString());
                _context2.next = 7;
                return _datadir2.default.validateOtaManifest(manifest, this.dataDir);

              case 7:
                valid = _context2.sent;

                if (valid) {
                  _context2.next = 10;
                  break;
                }

                return _context2.abrupt('return', null);

              case 10:
                return _context2.abrupt('return', manifest);

              case 13:
                _context2.prev = 13;
                _context2.t0 = _context2['catch'](0);

                _log2.default.error('OTA manifest cannot be parsed');
                return _context2.abrupt('return', null);

              case 17:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[0, 13]]);
      }));
      return function _loadManifest() {
        return ref.apply(this, arguments);
      };
    }()
  }, {
    key: 'latestDeviceFirmware',
    value: function latestDeviceFirmware(deviceId, version) {
      var deviceFirmware = null;
      this.manifest.firmwares.every(function (firmware) {
        if (firmware.devices.indexOf(deviceId) > -1) {
          deviceFirmware = firmware;
          return false;
        }

        return true;
      });

      if (!deviceFirmware) {
        return false;
      }

      return deviceFirmware;
    }
  }, {
    key: 'outdated',
    value: function outdated(deviceId, version) {
      var firmware = this.latestDeviceFirmware(deviceId, version);

      if (!firmware) {
        return false;
      }

      return firmware.version !== version;
    }
  }, {
    key: '_httpHandler',
    value: function () {
      var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(req, res) {
        var deviceData, deviceId, deviceVersion, freeBytes, firmware, firmwarePath, firmwareBuffer, firmwareSize, firmwareMd5;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (!(req.get('User-Agent') !== 'ESP8266-http-Update' || !req.get('x-ESP8266-free-space') || !req.get('x-ESP8266-version'))) {
                  _context3.next = 2;
                  break;
                }

                return _context3.abrupt('return', res.sendStatus(403));

              case 2:
                deviceData = req.get('x-ESP8266-version');
                deviceId = deviceData.split('=')[0];
                deviceVersion = deviceData.split(/=(.+)/)[1];
                freeBytes = parseInt(req.get('x-ESP8266-free-space'), 10);

                if (this.outdated(deviceId, deviceVersion)) {
                  _context3.next = 8;
                  break;
                }

                return _context3.abrupt('return', res.sendStatus(304));

              case 8:
                firmware = this.latestDeviceFirmware(deviceId, deviceVersion);
                firmwarePath = this.dataDir + '/ota/firmwares/' + firmware.name + '.bin';
                firmwareBuffer = undefined;
                _context3.prev = 11;
                _context3.next = 14;
                return fs.statAsync(firmwarePath).size;

              case 14:
                firmwareSize = _context3.sent;

                if (!(firmwareSize > freeBytes)) {
                  _context3.next = 18;
                  break;
                }

                _log2.default.error('OTA aborted, not enough free space on device', { deviceId: deviceId });
                return _context3.abrupt('return', res.sendStatus(304));

              case 18:
                _context3.next = 20;
                return fs.readFileAsync(firmwarePath);

              case 20:
                firmwareBuffer = _context3.sent;
                _context3.next = 27;
                break;

              case 23:
                _context3.prev = 23;
                _context3.t0 = _context3['catch'](11);

                _log2.default.error('OTA aborted, cannot access firmware ' + firmwarePath, { deviceId: deviceId });
                return _context3.abrupt('return', res.sendStatus(304));

              case 27:
                firmwareMd5 = (0, _md2.default)(firmwareBuffer);

                res.set('x-MD5', firmwareMd5);

                _log2.default.info('OTA update started', { deviceId: deviceId, version: firmware.version });

                return _context3.abrupt('return', res.sendFile(_path2.default.resolve(firmwarePath)));

              case 31:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this, [[11, 23]]);
      }));
      return function _httpHandler(_x, _x2) {
        return ref.apply(this, arguments);
      };
    }()
  }]);
  return OtaServer;
}(_server2.default);

// path resolve else with relative path express might cry
exports.default = OtaServer;