'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _validator = require('validator');

var _validator2 = _interopRequireDefault(_validator);

var _nodetypes = require('../../misc/nodetypes');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PropertyValidator = function () {
  function PropertyValidator() {
    (0, _classCallCheck3.default)(this, PropertyValidator);
  }

  (0, _createClass3.default)(PropertyValidator, [{
    key: '_getPropertyObject',
    value: function _getPropertyObject(type, property) {
      var typeObject = _nodetypes.types[type];
      if (!typeObject) {
        return false;
      }

      if (typeObject.alias) {
        typeObject = _nodetypes.types[typeObject.alias];
      }

      var propertyObject = typeObject.properties[property];
      if (!propertyObject) {
        return false;
      }

      return propertyObject;
    }
  }, {
    key: 'typeExists',
    value: function typeExists(type) {
      var typeObject = _nodetypes.types[type];
      if (!typeObject) {
        return false;
      }

      return true;
    }
  }, {
    key: 'canReceive',
    value: function canReceive(type, property, value) {
      var propertyObject = this._getPropertyObject(type, property);
      if (!propertyObject) {
        return false;
      }

      switch (propertyObject.type) {
        case 'boolean':
          return _validator2.default.isIn(value, ['true', 'false']);
        case 'float':
          return _validator2.default.isFloat(value);
        case 'percentage':
          return _validator2.default.isInt(value, { min: 0, max: 100 });
        case 'enum':
          return _validator2.default.isIn(value, propertyObject.oneOf);
        case 'blank':
          return true;
        case 'string':
          return true;
        default:
          return false;
      }
    }
  }, {
    key: 'canSend',
    value: function canSend(type, property, value) {
      if (!this.canReceive(type, property, value)) {
        return false;
      }

      var propertyObject = this._getPropertyObject(type, property);
      return propertyObject.settable;
    }
  }, {
    key: 'convertValue',
    value: function convertValue(type, property, value) {
      var propertyObject = this._getPropertyObject(type, property);
      if (!propertyObject) {
        return undefined;
      }

      if (!this.canReceive(type, property, value)) {
        throw new Error('Cannot convert value');
      }

      switch (propertyObject.type) {
        case 'boolean':
          return value === 'true';
        case 'float':
          return parseFloat(value);
        case 'percentage':
          return parseInt(value, 10);
        case 'enum':
          return value;
        case 'blank':
          return null;
        case 'string':
          return value;
        default:
          return undefined;
      }
    }
  }]);
  return PropertyValidator;
}();

exports.default = new PropertyValidator();